from rest_framework import serializers
from .models import Forum, ForumQuestion, Comment, ForumMembership
from userprofile_app.models import Profile, Industry
import json
from django.contrib.auth import get_user_model

User = get_user_model()

# class ForumSerializer(serializers.ModelSerializer):
#     categories = serializers.ListField(
#         required=True
#     )

#     class Meta:
#         model = Forum
#         fields = [
#             'user',
#             'title',
#             'categories',
#             'short_description',
#             'cover_image',
#             'hashtags',
#             'permission_to_join',
#             'invite_user_ids',
#         ]
    

#     def validate_categories(self, value):
#         if not value:
#             raise serializers.ValidationError("Categories cannot be empty.")

#         valid_categories = Industry.objects.filter(id__in=value[0])
#         if len(valid_categories) != len(value[0]):
#             raise serializers.ValidationError("Some of the categories are invalid.")
        
#         return value
    
#     def validate_invite_user_ids(self, value):
#         if value:
#             valid_user = User.objects.filter(id__in=value)
#             if len(valid_user) != len(value):
#                 raise serializers.ValidationError("Some of the users are invalid.")

#         return value


#     def create(self, validated_data):
#         categories = validated_data.pop('categories')
#         forum = Forum.objects.create(**validated_data)
#         for i in categories:
#             forum.categories.set(categories)
#         return forum

#     def update(self, instance, validated_data):
#         categories = validated_data.pop('categories', None)
#         for attr, value in validated_data.items():
#             setattr(instance, attr, value)
#         instance.save()
#         if categories is not None:
#             for i in categories:
#                 instance.categories.set(i)
#         return instance

class ForumSerializer(serializers.ModelSerializer):
    title = serializers.CharField(required=True)
    permission_to_join = serializers.ChoiceField(choices=Forum.PERMISSION_CHOICES, required=True)
    short_description = serializers.CharField(required=False, allow_blank=True, allow_null=True)
    cover_image = serializers.ImageField(required=False, allow_null=True)
    hashtags = serializers.ListField(child=serializers.CharField(), required=False, allow_null=True)
    invite_user_ids = serializers.ListField(child=serializers.IntegerField(), required=False, allow_null=True)
    class Meta:
        model = Forum
        fields = [
            'user',
            'title',
            'short_description',
            'cover_image',
            'hashtags',
            'permission_to_join',
            'invite_user_ids',
        ]


class ForumInviteSerializer(serializers.Serializer):
    forum_id = serializers.IntegerField()
    # userIds = serializers.ListField(child=serializers.IntegerField())


class ForumActionSerializer(serializers.Serializer):
    userId = serializers.IntegerField()
    forum_id = serializers.IntegerField()

class ForumRemoveUserSerializer(serializers.Serializer):
    removedBy = serializers.IntegerField()
    forum_id = serializers.IntegerField()
    userId = serializers.IntegerField()

    
class CommentSerializer(serializers.ModelSerializer):
    class Meta:
        model = Comment
        fields = ['id', 'question', 'user', 'comment', 'created_at']

class ForumQuestionSerializer(serializers.ModelSerializer):

    class Meta:
        model = ForumQuestion
        fields = ['id', 'forum', 'question', 'content', 'tags', 'created_at', 'user']
        read_only_fields = ['created_at', 'comments']

class ForumMemberSerializer(serializers.ModelSerializer):
    id = serializers.IntegerField(source='user.id', read_only=True)
    username = serializers.CharField(source='user.username', read_only=True)
    email = serializers.EmailField(source='user.email', read_only=True)
    joined_at = serializers.DateTimeField(source='created_at', read_only=True)

    class Meta:
        model = ForumMembership
        fields = ['id', 'username', 'email', 'joined_at']
    
class ForumViewSerializer(serializers.ModelSerializer):
    categories = serializers.SerializerMethodField()
    username = serializers.CharField(source='user.username', read_only=True)
    full_name = serializers.CharField(source='user.full_name', read_only=True)
    profile_image = serializers.ImageField(source='user.profile.profile_image', read_only=True)

    class Meta:
        model = Forum
        fields = [
            'id', 'user', 'title', 'categories', 'short_description','username','full_name',
            'cover_image', 'hashtags', 'permission_to_join', 'invite_user_ids','created_at','profile_image'
        ]

    def get_categories(self, obj):
        return [category.name for category in obj.categories.all()]

    